@@ -27,6 +27,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <glib.h>
+#include <errno.h>
 
 #include "intl.h"
 
@@ -54,6 +55,30 @@
 #define fchmod(f,m) (0)
 #endif
 
+static void read_connections(GList *objects, xmlNodePtr layer_node,
+			     GHashTable *objects_hash);
+static void GHFuncUnknownObjects(gpointer key,
+				 gpointer value,
+				 gpointer user_data);
+static GList *read_objects(xmlNodePtr objects, Layer *layer,
+			   GHashTable *objects_hash,
+			   const char *filename, Object *parent);
+static void hash_free_string(gpointer       key,
+			     gpointer       value,
+			     gpointer       user_data);
+static xmlNodePtr find_node_named (xmlNodePtr p, const char *name);
+static gboolean diagram_data_load(const char *filename, DiagramData *data,
+				  void* user_data);
+static gboolean write_objects(GList *objects, xmlNodePtr objects_node,
+			      GHashTable *objects_hash, int *obj_nr, 
+			      const char *filename);
+static gboolean write_connections(GList *objects, xmlNodePtr layer_node,
+				  GHashTable *objects_hash);
+static xmlDocPtr diagram_data_write_doc(DiagramData *data, const char *filename);
+static int diagram_data_raw_save(DiagramData *data, const char *filename);
+static int diagram_data_save(DiagramData *data, const char *filename);
+
+
 static void
 GHFuncUnknownObjects(gpointer key,
                      gpointer value,
@@ -67,7 +92,7 @@ GHFuncUnknownObjects(gpointer key,
 
 static GList *
 read_objects(xmlNodePtr objects, Layer *layer,
-             GHashTable *objects_hash,const char *filename)
+             GHashTable *objects_hash,const char *filename, Object *parent)
 {
   GList *list;
   ObjectType *type;
@@ -79,6 +104,7 @@ read_objects(xmlNodePtr objects, Layer *layer,
   int version;
   GHashTable* unknown_hash;
   GString*    unknown_str;
+  xmlNodePtr child_node;
 
   unknown_hash = g_hash_table_new(g_str_hash, g_str_equal);
   unknown_str  = g_string_new("Unknown types while reading diagram file");
@@ -117,14 +143,33 @@ read_objects(xmlNodePtr objects, Layer *layer,
         obj = type->ops->load(obj_node, version, filename);
         layer_add_object(layer,obj);
         list = g_list_append(list, obj);
-      
+
+        if (parent)
+        {
+          obj->parent = parent;
+          parent->children = g_list_append(parent->children, obj);
+        }
+
         g_hash_table_insert(objects_hash, g_strdup((char *)id), obj);
+
+        child_node = obj_node->children;
+
+        while(child_node)
+        {
+          if (strcmp(child_node->name, "children") == 0)
+          {
+	    GList *children_read = read_objects(child_node, layer, objects_hash, filename, obj);
+            list = g_list_concat(list, children_read);
+            break;
+          }
+          child_node = child_node->next;
+        }
       }
       if (typestr) xmlFree(typestr);
       if (id) xmlFree (id);
     } else if (strcmp(obj_node->name, "group")==0) {
       obj = group_create(read_objects(obj_node, layer,
-                                      objects_hash, filename));
+                                      objects_hash, filename, NULL));
       layer_add_object(layer,obj);
       
       list = g_list_append(list, obj);
@@ -165,7 +210,7 @@ read_connections(GList *objects, xmlNodePtr layer_node,
   obj_node = layer_node->xmlChildrenNode;
   while ((list != NULL) && (obj_node != NULL)) {
     Object *obj = (Object *) list->data;
-    
+
     while (obj_node && xmlIsBlankNode(obj_node)) obj_node = obj_node->next;
     if (!obj_node) break;
     
@@ -213,9 +258,26 @@ read_connections(GList *objects, xmlNodePtr layer_node,
 	  connection = connection->next;
 	}
       }
-      
     }
     
+    /* Now set up parent relationships. */
+    connections = obj_node->xmlChildrenNode;
+    while ((connections!=NULL) &&
+	   (strcmp(connections->name, "childnode")!=0))
+      connections = connections->next;
+    if (connections != NULL) {
+      tostr = xmlGetProp(connections, "parent");
+      if (tostr) {
+	obj->parent = g_hash_table_lookup(objects_hash, tostr);
+	if (obj->parent == NULL) {
+	  message_error(_("Can't find parent %s of %s object\n"), 
+			tostr, obj->type->name);
+	} else {
+	  obj->parent->children = g_list_prepend(obj->parent->children, obj);
+	}
+      }
+    }
+
     list = g_list_next(list);
     obj_node = obj_node->next;
   }
@@ -250,7 +312,8 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   AttributeNode attr;
   Layer *layer;
   xmlNsPtr namespace;
-  
+  gchar firstchar;
+
   if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
     message_error(_("You must specify a file, not a directory.\n"));
     return FALSE;
@@ -263,6 +326,14 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     return FALSE;
   }
 
+  if (read(fd, &firstchar, 1)) {
+    data->is_compressed = (firstchar != '<');
+  } else {
+    /* Couldn't read a single char?  Set to default. */
+    data->is_compressed = prefs.new_diagram.compress_save;
+  }
+  
+  /* Note that this closing and opening means we can't read from a pipe */
   close(fd);
 
   doc = xmlDiaParseFile(filename);
@@ -293,11 +364,16 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     find_node_named (doc->xmlRootNode->xmlChildrenNode, "diagramdata");
 
   /* Read in diagram data: */
-  data->bg_color = color_white;
+  data->bg_color = prefs.new_diagram.bg_color;
   attr = composite_find_attribute(diagramdata, "background");
   if (attr != NULL)
     data_color(attribute_first_data(attr), &data->bg_color);
 
+  data->pagebreak_color = prefs.new_diagram.pagebreak_color;
+  attr = composite_find_attribute(diagramdata, "pagebreak");
+  if (attr != NULL)
+    data_color(attribute_first_data(attr), &data->pagebreak_color);
+
   /* load paper information from diagramdata section */
   attr = composite_find_attribute(diagramdata, "paper");
   if (attr != NULL) {
@@ -386,6 +462,12 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     attr = composite_find_attribute(gridinfo, "visible_y");
     if (attr != NULL)
       data->grid.visible_y = data_int(attribute_first_data(attr));
+
+    data->grid.colour = prefs.new_diagram.grid_color;
+    attr = composite_find_attribute(diagramdata, "color");
+    if (attr != NULL)
+      data_color(attribute_first_data(attr), &data->grid.colour);
+
   }
   attr = composite_find_attribute(diagramdata, "guides");
   if (attr != NULL) {
@@ -448,7 +530,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 
     /* Read in all objects: */
     
-    list = read_objects(layer_node, layer, objects_hash, filename);
+    list = read_objects(layer_node, layer, objects_hash, filename, NULL);
     layer->objects = list;
     read_connections( list, layer_node, objects_hash);
 
@@ -458,7 +540,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   }
 
   data->active_layer = (Layer *) g_ptr_array_index(data->layers, 0);
-  
+
   xmlFreeDoc(doc);
   
   g_hash_table_foreach(objects_hash, hash_free_string, NULL);
@@ -475,7 +557,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   return TRUE;
 }
 
-static void
+static gboolean
 write_objects(GList *objects, xmlNodePtr objects_node,
 	      GHashTable *objects_hash, int *obj_nr, const char *filename)
 {
@@ -488,6 +570,12 @@ write_objects(GList *objects, xmlNodePtr objects_node,
   while (list != NULL) {
     Object *obj = (Object *) list->data;
 
+    if (g_hash_table_lookup(objects_hash, obj))
+    {
+      list = g_list_next(list);
+      continue;
+    }
+
     if IS_GROUP(obj) {
       group_node = xmlNewChild(objects_node, NULL, "group", NULL);
       write_objects(group_objects(obj), group_node,
@@ -496,7 +584,6 @@ write_objects(GList *objects, xmlNodePtr objects_node,
       obj_node = xmlNewChild(objects_node, NULL, "object", NULL);
     
       xmlSetProp(obj_node, "type", obj->type->name);
-      
       g_snprintf(buffer, 30, "%d", obj->type->version);
       xmlSetProp(obj_node, "version", buffer);
       
@@ -509,13 +596,28 @@ write_objects(GList *objects, xmlNodePtr objects_node,
       g_hash_table_insert(objects_hash, obj, GINT_TO_POINTER(*obj_nr));
       (*obj_nr)++;
       
+      /*
+      if (obj->can_parent && obj->children)
+      {
+	int res;
+	xmlNodePtr parent_node;
+        parent_node = xmlNewChild(obj_node, NULL, "children", NULL);
+        res = write_objects(obj->children, parent_node, objects_hash, obj_nr, filename);
+	if (!res) return FALSE;
+      }
+      */
     }
-    
+
     list = g_list_next(list);
   }
+  return TRUE;
 }
 
-static int
+/* Parents broke assumption that both objects and xml nodes are laid out
+ * linearly.  
+ */
+
+static gboolean
 write_connections(GList *objects, xmlNodePtr layer_node,
 		  GHashTable *objects_hash)
 {
@@ -572,13 +674,25 @@ write_connections(GList *objects, xmlNodePtr layer_node,
 	  g_snprintf(buffer, 30, "O%d",
 		     GPOINTER_TO_INT(g_hash_table_lookup(objects_hash,
 							 other_obj)));
+
 	  xmlSetProp(connection, "to", buffer);
 	  /* to what connection_point on that object */
 	  g_snprintf(buffer, 30, "%d", con_point_nr);
 	  xmlSetProp(connection, "connection", buffer);
+
+
 	}
       }
     }
+
+    if (obj->parent) {
+      Object *other_obj = obj->parent;
+      xmlNodePtr parent;
+      g_snprintf(buffer, 30, "O%d",
+		 GPOINTER_TO_INT(g_hash_table_lookup(objects_hash, other_obj)));
+      parent = xmlNewChild(obj_node, NULL, "childnode", NULL);
+      xmlSetProp(parent, "parent", buffer);
+    }
     
     list = g_list_next(list);
     obj_node = obj_node->next;
@@ -595,7 +709,7 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   xmlNodePtr pageinfo, gridinfo, guideinfo;
   xmlNodePtr layer_node;
   GHashTable *objects_hash;
-  int res;
+  gboolean res;
   int obj_nr;
   int i;
   Layer *layer;
@@ -616,6 +730,9 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   attr = new_attribute((ObjectNode)tree, "background");
   data_add_color(attr, &data->bg_color);
 
+  attr = new_attribute((ObjectNode)tree, "pagebreak");
+  data_add_color(attr, &data->pagebreak_color);
+
   attr = new_attribute((ObjectNode)tree, "paper");
   pageinfo = data_add_composite(attr, "paper");
   data_add_string(composite_add_attribute(pageinfo, "name"),
@@ -651,6 +768,10 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
 	       data->grid.visible_x);
   data_add_int(composite_add_attribute(gridinfo, "visible_y"),
 	       data->grid.visible_y);
+  attr = new_attribute((ObjectNode)tree, "color");
+  data_add_composite(gridinfo, "color");
+  data_add_color(attr, &data->grid.colour);
+  
 
   attr = new_attribute((ObjectNode)tree, "guides");
   guideinfo = data_add_composite(attr, "guides");
@@ -685,7 +806,7 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   }
   g_hash_table_destroy(objects_hash);
 
-  if (prefs.compress_save)
+  if (data->is_compressed)
     xmlSetDocCompressMode(doc, 9);
   else
     xmlSetDocCompressMode(doc, 0);
@@ -743,7 +864,7 @@ diagram_data_save(DiagramData *data, const char *filename)
   /* Now write the data in the temporary file name. */
 
   if (file==NULL) {
-    message_error(_("Couldn't open: '%s' for writing.\n"), tmpname);
+    message_error(_("Can't open output file %s: %s\n"), tmpname, strerror(errno));
     return FALSE;
   }
   fclose(file);
