@@ -20,21 +20,20 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
 #include "intl.h"
 #include "diagramdata.h"
-
+#include "diarenderer.h"
 #include "paper.h"
+#include "dynamic_obj.h"
 
 static const Rectangle invalid_extents = { -1.0,-1.0,-1.0,-1.0 };
+static void set_parent_layer(gpointer layer, gpointer object);
 
 DiagramData *
-new_diagram_data (void)
+new_diagram_data (NewDiagramData *prefs)
 {
 	DiagramData *data;
 	Layer *first_layer;
-	utfchar *utfstr;
    
 	data = g_new (DiagramData, 1);
    
@@ -43,26 +42,25 @@ new_diagram_data (void)
 	data->extents.top = 0.0; 
 	data->extents.bottom = 10.0; 
  
-	data->bg_color = color_white;
+	data->bg_color = prefs->bg_color;
+	data->pagebreak_color = prefs->pagebreak_color;
 
-	get_paper_info (&data->paper, -1);
+	get_paper_info (&data->paper, -1, prefs);
 
+	data->grid.dynamic = TRUE;
 	data->grid.width_x = 1.0;
 	data->grid.width_y = 1.0;
 	data->grid.visible_x = 1;
 	data->grid.visible_y = 1;
+	data->grid.colour = prefs->grid_color;
 
 	data->guides.nhguides = 0;
 	data->guides.hguides = NULL;
 	data->guides.nvguides = 0;
 	data->guides.vguides = NULL;
 
-#ifdef GTK_DOESNT_TALK_UTF8_WE_DO
-	utfstr = charconv_local8_to_utf8 (_("Background"));
-#else
-	utfstr = g_strdup (_("Background"));
-#endif
-	first_layer = new_layer (utfstr);
+	first_layer = new_layer(g_strdup(_("Background")),data);
+  
 	data->layers = g_ptr_array_new ();
 	g_ptr_array_add (data->layers, first_layer);
 	data->active_layer = first_layer;
@@ -70,6 +68,8 @@ new_diagram_data (void)
 	data->selected_count = 0;
 	data->selected = NULL;
   
+	data->is_compressed = prefs->compress_save; // Overridden by doc
+
 	return data;
 }
 
@@ -93,14 +93,15 @@ diagram_data_destroy(DiagramData *data)
 }
 
 Layer *
-new_layer(utfchar *name)
+new_layer(gchar *name, DiagramData *parent)
 {
   Layer *layer;
 
   layer = g_new(Layer, 1);
 
   layer->name = name;
-  
+
+  layer->parent_diagram = parent;
   layer->visible = TRUE;
 
   layer->objects = NULL;
@@ -133,7 +134,7 @@ data_raise_layer(DiagramData *data, Layer *layer)
       layer_nr = i;
   }
 
-  assert(layer_nr>=0);
+  g_assert(layer_nr>=0);
 
   if (layer_nr < data->layers->len-1) {
     tmp = g_ptr_array_index(data->layers, layer_nr+1);
@@ -155,7 +156,7 @@ data_lower_layer(DiagramData *data, Layer *layer)
       layer_nr = i;
   }
 
-  assert(layer_nr>=0);
+  g_assert(layer_nr>=0);
 
   if (layer_nr > 0) {
     tmp = g_ptr_array_index(data->layers, layer_nr-1);
@@ -170,6 +171,7 @@ void
 data_add_layer(DiagramData *data, Layer *layer)
 {
   g_ptr_array_add(data->layers, layer);
+  layer->parent_diagram = data;
   layer_update_extents(layer);
   data_update_extents(data);
 }
@@ -189,6 +191,7 @@ data_add_layer_at(DiagramData *data, Layer *layer, int pos)
     g_ptr_array_index(data->layers, pos) = layer;
   }
   
+  layer->parent_diagram = data;
   layer_update_extents(layer);
   data_update_extents(data);
 }
@@ -208,6 +211,7 @@ data_delete_layer(DiagramData *data, Layer *layer)
   if (data->active_layer == layer) {
     data_remove_all_selected(data);
   }
+  layer->parent_diagram = NULL;
   g_ptr_array_remove(data->layers, layer);
 
   if (data->active_layer == layer) {
@@ -385,7 +389,7 @@ data_get_sorted_selected_remove(DiagramData *data)
 }
 
 void
-data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
+data_render(DiagramData *data, DiaRenderer *renderer, Rectangle *update,
 	    ObjectRenderer obj_renderer /* Can be NULL */,
 	    gpointer gdata)
 {
@@ -393,7 +397,7 @@ data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
   int i;
   int active_layer;
 
-  if (!renderer->is_interactive) (renderer->ops->begin_render)(renderer);
+  if (!renderer->is_interactive) (DIA_RENDERER_GET_CLASS(renderer)->begin_render)(renderer);
   
   for (i=0; i<data->layers->len; i++) {
     layer = (Layer *) g_ptr_array_index(data->layers, i);
@@ -402,15 +406,15 @@ data_render(DiagramData *data, Renderer *renderer, Rectangle *update,
       layer_render(layer, renderer, update, obj_renderer, gdata, active_layer);
   }
   
-  if (!renderer->is_interactive) (renderer->ops->end_render)(renderer);
+  if (!renderer->is_interactive) (DIA_RENDERER_GET_CLASS(renderer)->end_render)(renderer);
 }
 
 static void
-normal_render(Object *obj, Renderer *renderer,
+normal_render(Object *obj, DiaRenderer *renderer,
 	      int active_layer,
 	      gpointer data)
 {
-  obj->ops->draw(obj, renderer);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_object(renderer, obj);
 }
 
 
@@ -418,7 +422,7 @@ int render_bounding_boxes = FALSE;
 
 /* If obj_renderer is NULL normal_render is used. */
 void
-layer_render(Layer *layer, Renderer *renderer, Rectangle *update,
+layer_render(Layer *layer, DiaRenderer *renderer, Rectangle *update,
 	     ObjectRenderer obj_renderer,
 	     gpointer data,
 	     int active_layer)
@@ -446,8 +450,8 @@ layer_render(Layer *layer, Renderer *renderer, Rectangle *update,
 	col.green = 0.0;
 	col.blue = 1.0;
 
-        renderer->ops->set_linewidth(renderer,0.01);
-	renderer->ops->draw_rect(renderer, &p1, &p2, &col);
+        DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer,0.01);
+	DIA_RENDERER_GET_CLASS(renderer)->draw_rect(renderer, &p1, &p2, &col);
       }
       (*obj_renderer)(obj, renderer, active_layer, data);
     }
@@ -456,6 +460,11 @@ layer_render(Layer *layer, Renderer *renderer, Rectangle *update,
   }
 }
 
+static void
+set_parent_layer(gpointer element, gpointer user_data) {
+  ((Object*)element)->parent_layer = (Layer*)user_data;
+}
+
 int
 layer_object_index(Layer *layer, Object *obj)
 {
@@ -466,30 +475,36 @@ void
 layer_add_object(Layer *layer, Object *obj)
 {
   layer->objects = g_list_append(layer->objects, (gpointer) obj);
+  set_parent_layer(obj, layer);
 }
 
 void
 layer_add_object_at(Layer *layer, Object *obj, int pos)
 {
   layer->objects = g_list_insert(layer->objects, (gpointer) obj, pos);
+  set_parent_layer(obj, layer);
 }
 
 void
 layer_add_objects(Layer *layer, GList *obj_list)
 {
   layer->objects = g_list_concat(layer->objects, obj_list);
+  g_list_foreach(obj_list, set_parent_layer, layer);
 }
 
 void
 layer_add_objects_first(Layer *layer, GList *obj_list)
 {
   layer->objects = g_list_concat(obj_list, layer->objects);
+  g_list_foreach(obj_list, set_parent_layer, layer);
 }
 
 void
 layer_remove_object(Layer *layer, Object *obj)
 {
   layer->objects = g_list_remove(layer->objects, obj);
+  dynobj_list_remove_object(obj);
+  set_parent_layer(obj, NULL);
 }
 
 void
@@ -502,6 +517,8 @@ layer_remove_objects(Layer *layer, GList *obj_list)
     layer->objects = g_list_remove(layer->objects, obj);
     
     obj_list = g_list_next(obj_list);
+    dynobj_list_remove_object(obj);
+    set_parent_layer(obj, NULL);
   }
 }
 
@@ -550,36 +567,54 @@ layer_find_objects_in_rectangle(Layer *layer, Rectangle *rect)
   return selected_list;
 }
 
+/** Find the object closest to the given point in the layer,
+ * no further away than maxdist, and not included in avoid.
+ * Stops it if finds an object that includes the point.
+ */
 Object *
-layer_find_closest_object(Layer *layer, Point *pos, real maxdist)
+layer_find_closest_object_except(Layer *layer, Point *pos,
+				 real maxdist, GList *avoid)
 {
   GList *l;
   Object *closest;
   Object *obj;
   real dist;
-  
+  GList *avoid_tmp;
+
   closest = NULL;
   
-  l = layer->objects;
-  while(l!=NULL) {
+  for (l = layer->objects; l!=NULL; l = g_list_next(l)) {
     obj = (Object *) l->data;
 
     /* Check bounding box here too. Might give speedup. */
     dist = obj->ops->distance_from(obj, pos);
 
-    if (dist<=maxdist) {
+    if (maxdist-dist > 0.00000001) {
+      for (avoid_tmp = avoid; avoid_tmp != NULL; avoid_tmp = avoid_tmp->next) {
+	if (avoid_tmp->data == obj) {
+	  goto NEXTOBJECT;
+	}
+      }
       closest = obj;
     }
-    
-    l = g_list_next(l);
+  NEXTOBJECT:
+  ;
   }
 
   return closest;
 }
 
+Object *
+layer_find_closest_object(Layer *layer, Point *pos, real maxdist)
+{
+  return layer_find_closest_object_except(layer, pos, maxdist, NULL);
+}
+
+
 real layer_find_closest_connectionpoint(Layer *layer,
 					ConnectionPoint **closest,
-					Point *pos)
+					Point *pos,
+					Object *notthis)
 {
   GList *l;
   Object *obj;
@@ -591,10 +626,10 @@ real layer_find_closest_connectionpoint(Layer *layer,
   
   *closest = NULL;
   
-  l = layer->objects;
-  while(l!=NULL) {
+  for (l = layer->objects; l!=NULL; l = g_list_next(l) ) {
     obj = (Object *) l->data;
 
+    if (obj == notthis) continue;
     for (i=0;i<obj->num_connections;i++) {
       cp = obj->connections[i];
       /* Note: Uses manhattan metric for speed... */
@@ -605,8 +640,7 @@ real layer_find_closest_connectionpoint(Layer *layer,
       }
     }
     
-    l = g_list_next(l);
-  }
+ }
 
   return mindist;
 }
@@ -646,8 +680,11 @@ layer_replace_object_with_list(Layer *layer, Object *remove_obj,
 
   list = g_list_find(layer->objects, remove_obj);
 
-  assert(list!=NULL);
-  
+  g_assert(list!=NULL);
+  set_parent_layer(remove_obj, NULL);
+  dynobj_list_remove_object(remove_obj);
+  g_list_foreach(insert_list, set_parent_layer, layer);
+
   if (list->prev == NULL) {
     layer->objects = insert_list;
   } else {
@@ -665,10 +702,15 @@ layer_replace_object_with_list(Layer *layer, Object *remove_obj,
 
 void layer_set_object_list(Layer *layer, GList *list)
 {
+  g_list_foreach(layer->objects, set_parent_layer, NULL);
+  g_list_foreach(layer->objects, dynobj_list_remove_object, NULL);
   g_list_free(layer->objects);
   layer->objects = list;
+  g_list_foreach(layer->objects, set_parent_layer, layer);
 }
 
-
-
-
+DiagramData *
+layer_get_parent_diagram(Layer *layer)
+{
+  return layer->parent_diagram;
+}
