@@ -17,46 +17,209 @@
  */
 
 #include <config.h>
+#include <string.h>
 #include "intl.h"
+#undef GTK_DISABLE_DEPRECATED /* ... */
 #include "widgets.h"
 #include "message.h"
+#include "dia_dirs.h"
+#include "arrows.h"
 
 #include <stdlib.h>
 #include <glib.h>
 #include <gdk/gdk.h>
-#include <gtk/gtkradiomenuitem.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
+#include <pango/pango.h>
+#include <stdio.h>
+#include <time.h>
 
-void dia_font_selector_set_styles (DiaFontSelector *fs, DiaFont *font);
-guchar **dia_font_selector_get_sorted_names();
-
-struct menudesc {
-  char *name;
-  int enum_value;
-};
-
-static void fill_menu(GtkMenu *menu, GSList **group, 
-                      const struct menudesc *menudesc)
-{
-  GtkWidget *menuitem;
-  const struct menudesc *md = menudesc;
-
-  while (md->name) {
-    menuitem = gtk_radio_menu_item_new_with_label (*group, md->name);
-    gtk_object_set_user_data(GTK_OBJECT(menuitem), 
-                             GINT_TO_POINTER(md->enum_value));
-    *group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-    gtk_menu_append (menu, menuitem);
-    gtk_widget_show (menuitem);
-
-    md++;
-  }
-}
+#include "diagtkfontsel.h"
 
 /************* DiaFontSelector: ***************/
 
-static GHashTable *font_nr_hashtable = NULL;
+/* New and improved font selector:  Contains the three standard fonts
+ * and an 'Other fonts...' entry that opens the font dialog.  The fonts
+ * selected in the font dialog are persistently added to the menu.
+ *
+ * +----------------+
+ * | Sans           |
+ * | Serif          |
+ * | Monospace      |
+ * | -------------- |
+ * | Bodini         |
+ * | CurlyGothic    |
+ * | OldWestern     |
+ * | -------------- |
+ * | Other fonts... |
+ * +----------------+
+ */
+
+typedef struct {
+  gchar *name;
+  PangoFontFamily *family;
+  time_t last_select;
+  int entry_nr;
+} FontSelectorEntry;
+
+/* Hash table from font name to FontSelectorEntry */
+static GHashTable *font_hash_table = NULL;
+static GList *menu_entry_list = NULL;
+
+static void dia_font_selector_dialog_callback(GtkWidget *widget, int id, gpointer data);
+static void dia_font_selector_menu_callback(GtkWidget *button, gpointer data);
+static void dia_font_selector_set_styles(DiaFontSelector *fs,
+			     FontSelectorEntry *fse,
+			     DiaFontStyle dia_style);
+
+static FontSelectorEntry *
+dia_font_selector_add_font(char *lowername, gchar *fontname,
+			   gboolean is_other_font) {
+  FontSelectorEntry *fse;
+  fse = g_new(FontSelectorEntry, 1);
+  /* Not using LRU yet, so each line is but a name */
+  fse->name = fontname;
+  fse->family = NULL;
+  fse->last_select = time(0);
+  fse->entry_nr = g_list_length(menu_entry_list)+4; /* Skip first entries */
+  g_hash_table_insert(font_hash_table, g_strdup(lowername), fse);
+  if (is_other_font) {
+    menu_entry_list = g_list_append(menu_entry_list, fontname);
+  } else {
+    if (!g_strcasecmp(fontname, "sans")) fse->entry_nr = 0;
+    if (!g_strcasecmp(fontname, "serif")) fse->entry_nr = 1;
+    if (!g_strcasecmp(fontname, "monospace")) fse->entry_nr = 2;
+  }
+  return fse;
+}
+
+static
+gboolean strcase_equal(gconstpointer s1, gconstpointer s2)
+{
+  return !(g_strcasecmp((char *)s1, (char *)s2));
+}
+
+static void
+dia_font_selector_read_persistence_file() {
+  gchar *file_contents;
+  gchar *persistence_name;
+  GError *error = NULL;
+
+  font_hash_table = g_hash_table_new(g_str_hash, strcase_equal);
+
+  dia_font_selector_add_font("sans", "Sans", FALSE);
+  dia_font_selector_add_font("serif", "Serif", FALSE);
+  dia_font_selector_add_font("monospace", "Monospace", FALSE);
+
+  persistence_name = dia_config_filename("font_menu");
+  
+  if (g_file_test(persistence_name, G_FILE_TEST_EXISTS) &&
+      g_file_get_contents(persistence_name, &file_contents, NULL, &error)) {
+    /* Should really use some macro for linebreak, but I can't find it */
+    gchar **lines = g_strsplit(file_contents, "\n", -1); 
+    int i;
+    for (i = 0; lines[i] != NULL; i++) {
+      gchar *lowername;
+      if (strlen(lines[i]) == 0) continue;
+      lowername = g_utf8_strdown(lines[i], -1);
+      dia_font_selector_add_font(lowername, lines[i], TRUE);
+      g_free(lowername);
+    }
+    g_free(file_contents);
+  }
+  if (error) g_error_free(error);
+  g_free(persistence_name);
+}
+
+static void
+dia_font_selector_write_persistence_file() {
+  gchar *persistence_name;
+  FILE *pfile;
+
+  persistence_name = dia_config_filename("font_menu");
+  if ((pfile = fopen(persistence_name, "wb")) != NULL) {
+    GList *entry;
+    for (entry = menu_entry_list; entry != NULL; entry = entry->next) {
+      fputs((gchar *)entry->data, pfile);
+      fputs("\n", pfile);
+    }
+    fclose(pfile);
+  }
+  g_free(persistence_name);
+}
+
+static GtkWidget *
+dia_font_selector_add_menu_item(gchar *label, GSList **group, GtkWidget *menu) 
+{
+  GtkWidget *menuitem;
+  menuitem = gtk_radio_menu_item_new_with_label (*group, label);
+  *group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
+  gtk_object_set_user_data(GTK_OBJECT(menuitem), label);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  gtk_widget_show (menuitem);
+  return menuitem;
+}
+
+static void
+dia_font_selector_build_font_menu(DiaFontSelector *fs) {
+  GtkWidget *menu;
+  GtkWidget *omenu;
+  GtkWidget *menuitem;
+  GSList *group;
+  GList *entry;
+
+  if (fs->font_omenu != NULL) {
+    gtk_option_menu_remove_menu(fs->font_omenu);
+    omenu = GTK_WIDGET(fs->font_omenu);
+  } else {
+    omenu = gtk_option_menu_new();
+    fs->font_omenu = GTK_OPTION_MENU(omenu);
+  }
+  menu = gtk_menu_new ();
+  fs->font_menu = GTK_MENU(menu);
+
+  group = NULL;
+
+  menuitem = dia_font_selector_add_menu_item("sans", &group, menu);
+  menuitem = dia_font_selector_add_menu_item("serif", &group, menu);
+  menuitem = dia_font_selector_add_menu_item("monospace", &group, menu);
+
+  menuitem = gtk_separator_menu_item_new();
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  gtk_widget_show (menuitem);
+
+  for (entry = menu_entry_list; entry != NULL; entry = entry->next) {
+    menuitem = dia_font_selector_add_menu_item((gchar *)entry->data, &group, menu);
+  }
+  menuitem = gtk_separator_menu_item_new();
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  gtk_widget_show (menuitem);
+
+  menuitem = gtk_menu_item_new_with_label(_("Other fonts..."));
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  gtk_object_set_user_data(GTK_OBJECT(menuitem), NULL);
+  gtk_widget_show (menuitem);
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (fs->font_omenu), menu);
+  gtk_widget_show(menu);
+  gtk_widget_show(omenu);
+
+  gtk_signal_connect(GTK_OBJECT(menu), "unmap", dia_font_selector_menu_callback, fs);
+}
+
+static FontSelectorEntry *
+dia_font_selector_get_new_font(DiaFontSelector *fs, gchar *fontname)
+{
+  gchar *lowername = g_utf8_strdown(fontname, -1);
+  FontSelectorEntry *fse = 
+    (FontSelectorEntry*)g_hash_table_lookup(font_hash_table, lowername);
+  if (fse == NULL) {
+    fse = dia_font_selector_add_font(lowername, fontname, TRUE);
+    dia_font_selector_build_font_menu(fs);
+    dia_font_selector_write_persistence_file();
+  }
+  g_free(lowername);
+  return fse;
+}
 
 static void
 dia_font_selector_class_init (DiaFontSelectorClass *class)
@@ -66,110 +229,15 @@ dia_font_selector_class_init (DiaFontSelectorClass *class)
   object_class = (GtkObjectClass*) class;
 }
 
-#ifdef HAVE_FREETYPE
-/* This function is stolen from dillo.
- */
-static void
-dia_font_selector_scroll_popup(GtkWidget *widget)
-{
-  /*
-   * todo:
-   *   1) Scrolling menues should rather be the task of Gtk+. This is
-   *      a hack, and I don't know if it does not break anything.
-   *   2) It could be improved, e.g. a timeout could be added for
-   *      better mouse navigation.
-    */
-  int y, h, mx, my, sh;
-
-  y = widget->allocation.y;
-  h = widget->allocation.height;
-  gdk_window_get_geometry (widget->parent->parent->window,
-			   &mx, &my, NULL, NULL, NULL);
-  sh = gdk_screen_height ();
-
-  if (y + my < 0)
-    gdk_window_move (widget->parent->parent->window, mx, - y + 1);
-  else if (y + my > sh - h)
-    gdk_window_move (widget->parent->parent->window, mx, sh - h - y - 1);
-}
-
-static void
-dia_font_selector_font_selected(GtkWidget *button, gpointer data)
-{
-  DiaFontSelector *fs = DIAFONTSELECTOR(data);
-
-  GtkWidget *active = gtk_menu_get_active(fs->font_menu);
-  char *fontname = (char *)gtk_object_get_user_data(GTK_OBJECT(active));
-  dia_font_selector_set_styles(fs, font_getfont(fontname));
-}
-
-struct select_pair { guchar **array; int counter; };
-
-static void
-dia_font_selector_collect_names(gpointer key, gpointer value,
-				gpointer userdata) {
-  struct select_pair *pair = (struct select_pair *)userdata;
-  pair->array[pair->counter++] = (guchar *)key;
-}
-
-static int
-dia_font_selector_sort_string(const void *s1, const void *s2) {
-  return strcasecmp(*(char **)s1, *(char **)s2);
-}
-
-static guchar **
-dia_font_selector_get_fonts_sorted() {
-  struct select_pair pair;
-  int num_fonts;
-
-  num_fonts = g_hash_table_size(freetype_fonts);
-  pair.array = (guchar **)g_malloc(sizeof(guchar*)*num_fonts);
-  pair.counter = 0;
-  g_hash_table_foreach(freetype_fonts, dia_font_selector_collect_names,
-		       (gpointer)&pair);
-  
-  qsort(pair.array, num_fonts, sizeof(guchar *), dia_font_selector_sort_string);
-  return pair.array;
-}
-
-#endif
-
 static void
 dia_font_selector_init (DiaFontSelector *fs)
 {
-#ifdef HAVE_FREETYPE
-  /* Go through hash table and build menu */
   GtkWidget *menu;
   GtkWidget *omenu;
-  int i;
-  guchar **fontnames;
-  int num_fonts;
-
-  omenu = gtk_option_menu_new();
-  fs->font_omenu = GTK_OPTION_MENU(omenu);
-  menu = gtk_menu_new ();
-  fs->font_menu = GTK_MENU(menu);
-
-  fontnames = dia_font_selector_get_fonts_sorted();
-  num_fonts = g_hash_table_size(freetype_fonts);
-
-  for (i = 0; i < num_fonts; i++) {
-    GtkWidget *menuitem;
-    menuitem = gtk_menu_item_new_with_label (fontnames[i]);
-    gtk_object_set_user_data(GTK_OBJECT(menuitem), fontnames[i]);
-    gtk_signal_connect(GTK_OBJECT(menuitem), "select", dia_font_selector_scroll_popup, NULL);
-    gtk_menu_append (GTK_MENU (menu), menuitem);
-    gtk_widget_show (menuitem);
-  }
-
-  g_free(fontnames);
-
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (fs->font_omenu), menu);
-  gtk_widget_show(menu);
-  gtk_widget_show(omenu);
-
-  gtk_signal_connect(GTK_OBJECT(menu), "unmap", dia_font_selector_font_selected, fs);
 
+  dia_font_selector_build_font_menu(fs);
+  
+  /* Now build the style menu button */
   omenu = gtk_option_menu_new();
   fs->style_omenu = GTK_OPTION_MENU(omenu);
   menu = gtk_menu_new ();
@@ -181,46 +249,12 @@ dia_font_selector_init (DiaFontSelector *fs)
 
   gtk_box_pack_start_defaults(GTK_BOX(fs), GTK_WIDGET(fs->font_omenu));
   gtk_box_pack_start_defaults(GTK_BOX(fs), GTK_WIDGET(fs->style_omenu));
-#else
-  GtkWidget *menu;
-  GtkWidget *submenu;
-  GtkWidget *menuitem;
-  GSList *group;
-  GList *list;
-  char *fontname;
-  
-  menu = gtk_menu_new ();
-  fs->font_menu = GTK_MENU(menu);
-  submenu = NULL;
-  group = NULL;
-
-  list = font_names;
-  while (list != NULL) {
-    fontname = (char *) list->data;
-    menuitem = gtk_radio_menu_item_new_with_label (group, fontname);
-    gtk_object_set_user_data(GTK_OBJECT(menuitem), fontname);
-    group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-    gtk_menu_append (GTK_MENU (menu), menuitem);
-    gtk_widget_show (menuitem);
-
-    list = g_list_next(list);
-  }
-  
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (fs), menu);
-#endif
 }
 
 guint
 dia_font_selector_get_type        (void)
 {
   static guint dfs_type = 0;
-  GList *list;
-  char *fontname;
-  int i;
-#ifdef HAVE_FREETYPE
-  guchar **fontnames;
-  int num_fonts;
-#endif
 
   if (!dfs_type) {
     GtkTypeInfo dfs_info = {
@@ -229,43 +263,14 @@ dia_font_selector_get_type        (void)
       sizeof (DiaFontSelectorClass),
       (GtkClassInitFunc) dia_font_selector_class_init,
       (GtkObjectInitFunc) dia_font_selector_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL
     };
     
-#ifdef HAVE_FREETYPE
     dfs_type = gtk_type_unique (gtk_hbox_get_type (), &dfs_info);
-#else
-    dfs_type = gtk_type_unique (gtk_option_menu_get_type (), &dfs_info);
-#endif
 
-    /* Init the font hash_table: */
-    font_nr_hashtable = g_hash_table_new(g_str_hash, g_str_equal);
-
-#ifdef HAVE_FREETYPE
-    num_fonts = g_hash_table_size(freetype_fonts);
-    fontnames = dia_font_selector_get_fonts_sorted();
-    for (i = 0; i < num_fonts; i++) {
-      /* Notice adding 1, to be able to discern from NULL return value */
-      g_hash_table_insert(font_nr_hashtable,
-			  fontnames[i], GINT_TO_POINTER(i+1));
-    }
-    g_free(fontnames);
-#else
-    /* Notice starting at 1, to be able to discern from NULL return value */
-    i=1;
-    list = font_names;
-    while (list != NULL) {
-      fontname = (char *) list->data;
-      
-      g_hash_table_insert(font_nr_hashtable,
-			  fontname,
-			  GINT_TO_POINTER(i));
-  
-      list = g_list_next(list);
-      i++;
-    }
-#endif
+    dia_font_selector_read_persistence_file();
   }
   
   return dfs_type;
@@ -277,62 +282,208 @@ dia_font_selector_new ()
   return GTK_WIDGET ( gtk_type_new (dia_font_selector_get_type ()));
 }
 
-#ifdef HAVE_FREETYPE
+static PangoFontFamily *
+dia_font_selector_get_family_from_name(GtkWidget *widget, gchar *fontname)
+{
+  PangoFontFamily **families;
+  int n_families,i;
+  DiaFontFamily diafamily = DIA_FONT_FAMILY_ANY;
+    
+  pango_context_list_families (dia_font_get_context(),
+			       &families, &n_families);
+  /* Doing it the slow way until I find a better way */
+  for (i = 0; i < n_families; i++) {
+    if (!(g_strcasecmp(pango_font_family_get_name(families[i]), fontname)))
+      return families[i];
+  }
+  g_warning(_("Couldn't find font family for %s\n"), fontname);
+  return NULL;
+}
+
+/** Called when a font selection dialog is closed */
+static void
+dia_font_selector_dialog_callback(GtkWidget *widget, int id, gpointer data)
+{
+  DiaGtkFontSelectionDialog *fs = (DiaGtkFontSelectionDialog*)widget;
+  DiaFontSelector *dfs = (DiaFontSelector*)data;
+  gchar *fontname;
+  PangoFontDescription *pfd;
+  DiaFont *diafont;
+
+  switch (id) {
+  case GTK_RESPONSE_OK:
+    fontname = dia_gtk_font_selection_dialog_get_font_name(fs);
+    pfd = pango_font_description_from_string(fontname);
+    fontname = pango_font_description_get_family(pfd);
+    
+    dia_font_selector_get_new_font(dfs, fontname);
+    diafont = dia_font_new(fontname, DIA_FONT_NORMAL | DIA_FONT_WEIGHT_NORMAL, 1.0);
+    dia_font_selector_set_font(dfs, diafont);
+    break;
+  case GTK_RESPONSE_NONE:
+  case GTK_RESPONSE_CANCEL:
+  default:
+    /* noop */;
+  }
+  gtk_widget_hide(GTK_WIDGET(fs));
+}
+
+static void
+dia_font_selector_menu_callback(GtkWidget *button, gpointer data)
+{
+  DiaFontSelector *fs = DIAFONTSELECTOR(data);
+  gchar *fontname;
+
+  GtkWidget *active = gtk_menu_get_active(fs->font_menu);
+  if (active == NULL) return;
+  fontname = (gchar *)gtk_object_get_user_data(GTK_OBJECT(active));
+  if (fontname == NULL) {
+    /* We hit the Other fonts... entry */
+    GtkWidget *fsd = dia_gtk_font_selection_dialog_new(_("Select font"));
+    dia_gtk_font_selection_dialog_set_context
+      (DIA_GTK_FONT_SELECTION_DIALOG(fsd),
+      dia_font_get_context());
+    gtk_signal_connect(GTK_OBJECT(fsd), "response", 
+		       dia_font_selector_dialog_callback, data);
+    if (fs->textsample != NULL)
+      dia_gtk_font_selection_dialog_set_preview_text(fs, fs->textsample);
+    gtk_widget_show(fsd);
+  } else {
+    FontSelectorEntry *fse;
+    gchar *lowername = g_utf8_strdown(fontname, -1);
+    fse = (FontSelectorEntry*)g_hash_table_lookup(font_hash_table, lowername);
+    g_free(lowername);
+    dia_font_selector_set_styles(fs, fse, -1);
+  }
+}
+
+static char *style_labels[] = {
+  "Normal",
+  "Oblique",
+  "Italic",
+  "Ultralight",
+  "Ultralight-Oblique",
+  "Ultralight-Italic",
+  "Light",
+  "Light-Oblique",
+  "Light-Italic",
+  "Medium",
+  "Medium-Oblique",
+  "Medium-Italic",
+  "Demibold",
+  "Demibold-Oblique",
+  "Demibold-Italic",
+  "Bold",
+  "Bold-Oblique",
+  "Bold-Italic",
+  "Ultrabold",
+  "Ultrabold-Oblique",
+  "Ultrabold-Italic",
+  "Heavy",
+  "Heavy-Oblique",
+  "Heavy-Italic"
+};
+
 void
-dia_font_selector_set_styles(DiaFontSelector *fs, DiaFont *font)
+dia_font_selector_set_styles(DiaFontSelector *fs, FontSelectorEntry *fse,
+                             DiaFontStyle dia_style)
 {
   int i=0, select = 0;
-  GList *style_list = font->family->diafonts;
-
+  PangoFontFace **faces;
+  int nfaces;
   GtkWidget *menu = gtk_menu_new();
+  long stylebits = 0;
+  int menu_item_nr = 0;
+  GSList *group = NULL;
 
-  while (style_list != NULL) {
-    DiaFont *style_font = (DiaFont *)style_list->data;
-    GtkWidget *menuitem = gtk_menu_item_new_with_label (style_font->style);
-    gtk_object_set_user_data(GTK_OBJECT(menuitem), style_font->style);
-    if (!strcmp(style_font->style, font->style))
-      select = i;
-    i++;
+  if (fse->family == NULL) {
+    PangoFontFamily *pff;
+    pff = dia_font_selector_get_family_from_name(fs, fse->name);
+    fse->family = pff;
+    if (fse->family == NULL)
+      return;
+  }
+
+  pango_font_family_list_faces(fse->family, &faces, &nfaces);
+
+  for (i = 0; i < nfaces; i++) {
+    PangoFontDescription *pfd = pango_font_face_describe(faces[i]);
+    PangoStyle style = pango_font_description_get_style(pfd);
+    PangoWeight weight = pango_font_description_get_weight(pfd);
+    /**
+     * This is a quick and dirty way to pick the styles present,
+     * sort them and avoid duplicates.  
+     * We set a bit for each style present, bit (weight*3+style)
+     * From style_labels, we pick #(weight*3+style)
+     * where weight and style are the Dia types.  
+     */
+    /* Account for DIA_WEIGHT_NORMAL hack */
+    int weightnr = (weight-200)/100;
+    if (weightnr < 2) weightnr ++;
+    else if (weightnr == 2) weightnr = 0;
+    stylebits |= 1 << (3*weightnr + style);
+    pango_font_description_free(pfd);
+  }
+
+  for (i = DIA_FONT_NORMAL; i <= (DIA_FONT_HEAVY | DIA_FONT_ITALIC); i+=4) {
+    GtkWidget *menuitem;
+    /**
+     * bad hack continued ...
+     */
+    int weight = DIA_FONT_STYLE_GET_WEIGHT(i) >> 4;
+    int slant = DIA_FONT_STYLE_GET_SLANT(i) >> 2;
+    if (DIA_FONT_STYLE_GET_SLANT(i) > DIA_FONT_ITALIC) continue;
+    if (!(stylebits & (1 << (3*weight + slant)))) continue;
+    menuitem = gtk_radio_menu_item_new_with_label (group, style_labels[3*weight+slant]);
+    group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
+    gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(i));
+    if (dia_style == i) {
+      select = menu_item_nr;
+    }
+    menu_item_nr++;
     gtk_menu_append (GTK_MENU (menu), menuitem);
     gtk_widget_show (menuitem);
-    style_list = g_list_next(style_list);
   }
   gtk_widget_show(menu);
-  /* Need to dealloc the menu, methinks */
   gtk_option_menu_remove_menu(fs->style_omenu);
   gtk_option_menu_set_menu(fs->style_omenu, menu);
   fs->style_menu = GTK_MENU(menu);
   gtk_option_menu_set_history(GTK_OPTION_MENU(fs->style_omenu), select);
   gtk_menu_set_active(fs->style_menu, select);
+  gtk_widget_set_sensitive(GTK_WIDGET(fs->style_omenu), menu_item_nr > 1);
+  gtk_check_menu_item_set_active(gtk_menu_get_active(fs->style_menu), TRUE);
 }
-#endif
 
+/* API functions */
+/** Set a string to be used for preview in the GTK font selector dialog.
+ * The start of this string will be copied.
+ */
+void
+dia_font_selector_set_preview(DiaFontSelector *fs, gchar *text) {
+  if (fs->textsample != NULL) g_free(fs->textsample);
+  fs->textsample = g_strndup(text, 15);
+}
+
+/** Set the current font to be shown in the font selector.
+ */
 void
 dia_font_selector_set_font(DiaFontSelector *fs, DiaFont *font)
 {
-  void *font_nr_ptr;
   int font_nr;
+  DiaFontStyle style = dia_font_get_style(font);
+  GtkMenuItem *menuitem;
+  FontSelectorEntry *fse;
+  gchar *fontname = dia_font_get_family(font);
 
-  font_nr_ptr = g_hash_table_lookup(font_nr_hashtable,
-				    font->name);
+  fse = dia_font_selector_get_new_font(fs, fontname);
+  fse->last_select = time(0);
+  dia_font_selector_set_styles(fs, fse, dia_font_get_style(font));
+
+  font_nr = fse->entry_nr;
 
-  if (font_nr_ptr==NULL) {
-    message_error("Trying to set invalid font %s!\n", font->name);
-    font_nr = 0;
-  } else {
-    /* Notice subtracting 1, to be able to discern from NULL return value */
-    font_nr = GPOINTER_TO_INT(font_nr_ptr)-1;
-  }
-  
-#ifdef HAVE_FREETYPE
   gtk_option_menu_set_history(GTK_OPTION_MENU(fs->font_omenu), font_nr);
   gtk_menu_set_active(fs->font_menu, font_nr);
-
-  dia_font_selector_set_styles(fs, font);
-#else
-  gtk_option_menu_set_history(GTK_OPTION_MENU(fs), font_nr);
-  gtk_menu_set_active(fs->font_menu, font_nr);
-#endif
+  gtk_check_menu_item_set_active(gtk_menu_get_active(fs->font_menu), TRUE);
 }
 
 DiaFont *
@@ -340,20 +491,19 @@ dia_font_selector_get_font(DiaFontSelector *fs)
 {
   GtkWidget *menuitem;
   char *fontname;
-#ifdef HAVE_FREETYPE
-  char *stylename;
-#endif
+  DiaFontStyle style;
+  FontSelectorEntry *fse;
+  gchar *lowername;
 
   menuitem = gtk_menu_get_active(fs->font_menu);
-  fontname = gtk_object_get_user_data(GTK_OBJECT(menuitem));
-
-#ifdef HAVE_FREETYPE
+  fontname = (gchar *)gtk_object_get_user_data(GTK_OBJECT(menuitem));
+  lowername = g_utf8_strdown(fontname, -1);
+  fse = (FontSelectorEntry*)g_hash_table_lookup(font_hash_table, lowername);
+  g_free(lowername);
+  fse->last_select = time(0);
   menuitem = gtk_menu_get_active(fs->style_menu);
-  stylename = gtk_object_get_user_data(GTK_OBJECT(menuitem));
-  return font_getfont_with_style(fontname, stylename);
-#else
-  return font_getfont(fontname);
-#endif
+  style = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+  return dia_font_new(fontname,style,1.0);
 }
 
 /************* DiaAlignmentSelector: ***************/
@@ -381,19 +531,19 @@ dia_alignment_selector_init (DiaAlignmentSelector *fs)
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Left"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(ALIGN_LEFT));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
 
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Center"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(ALIGN_CENTER));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
 
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Right"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(ALIGN_RIGHT));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
   
   gtk_menu_set_active(GTK_MENU (menu), DEFAULT_ALIGNMENT);
@@ -412,8 +562,9 @@ dia_alignment_selector_get_type        (void)
       sizeof (DiaAlignmentSelectorClass),
       (GtkClassInitFunc) dia_alignment_selector_class_init,
       (GtkObjectInitFunc) dia_alignment_selector_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     
     dfs_type = gtk_type_unique (gtk_option_menu_get_type (), &dfs_info);
@@ -501,31 +652,31 @@ dia_line_style_selector_init (DiaLineStyleSelector *fs)
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Solid"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(LINESTYLE_SOLID));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
 
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Dashed"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(LINESTYLE_DASHED));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
 
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Dash-Dot"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(LINESTYLE_DASH_DOT));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
 
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Dash-Dot-Dot"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(LINESTYLE_DASH_DOT_DOT));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
   
   menuitem = gtk_radio_menu_item_new_with_label (group, _("Dotted"));
   gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(LINESTYLE_DOTTED));
   group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
   
   gtk_menu_set_active(GTK_MENU (menu), DEFAULT_LINESTYLE);
@@ -571,8 +722,9 @@ dia_line_style_selector_get_type        (void)
       sizeof (DiaLineStyleSelectorClass),
       (GtkClassInitFunc) dia_line_style_selector_class_init,
       (GtkObjectInitFunc) dia_line_style_selector_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     
     dfs_type = gtk_type_unique (gtk_vbox_get_type (), &dfs_info);
@@ -599,7 +751,7 @@ dia_line_style_selector_get_linestyle(DiaLineStyleSelector *fs,
   align = gtk_object_get_user_data(GTK_OBJECT(menuitem));
   *ls = GPOINTER_TO_INT(align);
   if (dl!=NULL) {
-    *dl = gtk_spin_button_get_value_as_float(fs->dashlength);
+    *dl = gtk_spin_button_get_value(fs->dashlength);
   }
 }
 
@@ -609,6 +761,7 @@ dia_line_style_selector_set_linestyle (DiaLineStyleSelector *as,
 {
   gtk_menu_set_active(GTK_MENU (as->linestyle_menu), linestyle);
   gtk_option_menu_set_history (GTK_OPTION_MENU(as->omenu), linestyle);
+  gtk_check_menu_item_set_active (gtk_menu_get_active(GTK_MENU(as->linestyle_menu)), TRUE);
   set_linestyle_sensitivity(DIALINESTYLESELECTOR(as));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(as->dashlength), dashlength);
 }
@@ -670,16 +823,14 @@ dia_color_selector_draw_area(GtkWidget          *area,
 static void
 dia_color_selector_ok(GtkWidget *widget, DiaColorSelector *cs)
 {
-  gdouble gcol[3];
+  GdkColor gcol;
   Color col;
 
-  gtk_color_selection_get_color(
+  gtk_color_selection_get_current_color(
 	GTK_COLOR_SELECTION(
 	    GTK_COLOR_SELECTION_DIALOG(cs->col_sel)->colorsel),
-	gcol);
-  col.red = gcol[0];
-  col.green = gcol[1];
-  col.blue = gcol[2];
+	&gcol);
+  GDK_COLOR_TO_DIA(gcol, col);
 
   dia_color_selector_set_color(cs, &col);
   gtk_widget_hide(GTK_WIDGET(cs->col_sel));
@@ -691,30 +842,36 @@ dia_color_selector_pressed(GtkWidget *widget)
   GtkColorSelectionDialog *dialog;
   DiaColorSelector *cs = DIACOLORSELECTOR(widget);
   
-  gdouble col[3];
+  GdkColor col;
 
   if (cs->col_sel == NULL) {
     cs->col_sel = gtk_color_selection_dialog_new(_("Select color"));
     dialog = GTK_COLOR_SELECTION_DIALOG(cs->col_sel);
+
+    gtk_color_selection_set_has_palette (
+	GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel),
+	TRUE);
+
     gtk_widget_hide(dialog->help_button);
     
     gtk_signal_connect (GTK_OBJECT (dialog->ok_button), "clicked",
 			(GtkSignalFunc) dia_color_selector_ok,
 			cs);
+    gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+			(GtkSignalFunc) gtk_widget_destroyed,
+			&cs->col_sel);
     
     gtk_signal_connect_object(GTK_OBJECT (dialog->cancel_button), "clicked",
 			      (GtkSignalFunc) gtk_widget_hide,
 			      GTK_OBJECT(dialog));
   }
 
-  col[0] = cs->col.red;
-  col[1] = cs->col.green;
-  col[2] = cs->col.blue;
+  DIA_COLOR_TO_GDK(cs->col, col);
 
-  gtk_color_selection_set_color(
+  gtk_color_selection_set_current_color(
 	GTK_COLOR_SELECTION(
 	    GTK_COLOR_SELECTION_DIALOG(cs->col_sel)->colorsel),
-	col);
+	&col);
   gtk_widget_show(cs->col_sel);
 }
 
@@ -755,8 +912,9 @@ dia_color_selector_get_type        (void)
       sizeof (DiaColorSelectorClass),
       (GtkClassInitFunc) dia_color_selector_class_init,
       (GtkObjectInitFunc) dia_color_selector_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     
     dfs_type = gtk_type_unique (gtk_button_get_type (), &dfs_info);
@@ -792,16 +950,12 @@ dia_color_selector_set_color (DiaColorSelector *cs,
   }
 
   if (cs->col_sel != NULL) {
-    gdouble col[3];
+    DIA_COLOR_TO_GDK(cs->col, col);
 
-    col[0] = cs->col.red;
-    col[1] = cs->col.green;
-    col[2] = cs->col.blue;
-
-    gtk_color_selection_set_color(
+    gtk_color_selection_set_current_color(
 	GTK_COLOR_SELECTION(
 	    GTK_COLOR_SELECTION_DIALOG(cs->col_sel)->colorsel),
-	col);
+	&col);
   }
 }
 
@@ -835,29 +989,25 @@ arrow_type_change_callback(GtkObject *as, gboolean arg1, gpointer data)
   set_size_sensitivity(DIAARROWSELECTOR(as));
 }
 
-static struct menudesc arrow_types[] =
-{{N_("None"),ARROW_NONE},
- {N_("Lines"),ARROW_LINES},
- {N_("Hollow Triangle"),ARROW_HOLLOW_TRIANGLE},
- {N_("Filled Triangle"),ARROW_FILLED_TRIANGLE},
- {N_("Unfilled Triangle"),ARROW_UNFILLED_TRIANGLE},
- {N_("Filled Diamond"),ARROW_FILLED_DIAMOND},
- {N_("Half Head"),ARROW_HALF_HEAD},
- {N_("Slashed Cross"),ARROW_SLASHED_CROSS},
- {N_("Filled Ellipse"),ARROW_FILLED_ELLIPSE},
- {N_("Hollow Ellipse"),ARROW_HOLLOW_ELLIPSE},
- {N_("Filled Dot"),ARROW_FILLED_DOT},
- {N_("Dimension Origin"),ARROW_DIMENSION_ORIGIN},
- {N_("Blanked Dot"),ARROW_BLANKED_DOT},
- {N_("Double Hollow triangle"),ARROW_DOUBLE_HOLLOW_TRIANGLE},
- {N_("Double Filled triangle"),ARROW_DOUBLE_FILLED_TRIANGLE},
- {N_("Filled Box"),ARROW_FILLED_BOX},
- {N_("Blanked Box"),ARROW_BLANKED_BOX},
- {N_("Slashed"),ARROW_SLASH_ARROW},
- {N_("Integral Symbol"),ARROW_INTEGRAL_SYMBOL},
- {N_("Crow Foot"),ARROW_CROW_FOOT},
- {N_("Cross"),ARROW_CROSS},
- {NULL,0}};
+/* This is actually quite general, but only used here */
+static void dia_arrow_fill_menu(GtkMenu *menu, GSList **group, 
+                      const struct menudesc *menudesc)
+{
+  GtkWidget *menuitem;
+  const struct menudesc *md = menudesc;
+
+  while (md->name) {
+    menuitem = gtk_radio_menu_item_new_with_label (*group, md->name);
+    gtk_object_set_user_data(
+			     GTK_OBJECT(menuitem), 
+			     GINT_TO_POINTER(((struct menudesc*)md)->enum_value));
+    *group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
+    gtk_menu_shell_append (GTK_MENU_SHELL(menu), menuitem);
+    gtk_widget_show (menuitem);
+
+    md++;
+  }
+}
 
 
 static void
@@ -881,7 +1031,7 @@ dia_arrow_selector_init (DiaArrowSelector *as)
   submenu = NULL;
   group = NULL;
 
-  fill_menu(GTK_MENU(menu),&group,arrow_types);
+  dia_arrow_fill_menu(GTK_MENU(menu),&group,arrow_types);
 
   gtk_menu_set_active(GTK_MENU (menu), DEFAULT_ARROW);
   gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
@@ -937,8 +1087,9 @@ dia_arrow_selector_get_type        (void)
       sizeof (DiaArrowSelectorClass),
       (GtkClassInitFunc) dia_arrow_selector_class_init,
       (GtkObjectInitFunc) dia_arrow_selector_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     
     dfs_type = gtk_type_unique (gtk_vbox_get_type (), &dfs_info);
@@ -983,6 +1134,7 @@ dia_arrow_selector_set_arrow (DiaArrowSelector *as,
   }
   gtk_menu_set_active(GTK_MENU (as->arrow_type_menu), arrow_type_index);
   gtk_option_menu_set_history (GTK_OPTION_MENU(as->omenu), arrow_type_index);
+  gtk_check_menu_item_set_active (gtk_menu_get_active(GTK_MENU(as->arrow_type_menu)), TRUE);
   set_size_sensitivity(as);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(as->width), arrow.width);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(as->length), arrow.length);
@@ -1041,6 +1193,10 @@ dia_file_selector_browse_pressed(GtkWidget *widget, gpointer data)
 			(GtkSignalFunc) dia_file_selector_ok,
 			dialog);
     
+    gtk_signal_connect (GTK_OBJECT (fs->dialog), "destroy",
+			GTK_SIGNAL_FUNC (gtk_widget_destroyed),
+			&fs->dialog);
+    
     gtk_signal_connect_object(GTK_OBJECT (dialog->cancel_button), "clicked",
 			      (GtkSignalFunc) gtk_widget_hide,
 			      GTK_OBJECT(dialog));
@@ -1083,8 +1239,9 @@ dia_file_selector_get_type (void)
       sizeof (DiaFileSelectorClass),
       (GtkClassInitFunc) dia_file_selector_class_init,
       (GtkObjectInitFunc) dia_file_selector_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     
     dfs_type = gtk_type_unique (gtk_hbox_get_type (), &dfs_info);
@@ -1107,10 +1264,9 @@ dia_file_selector_set_file(DiaFileSelector *fs, gchar *file)
   gtk_entry_set_text(GTK_ENTRY(fs->entry), file);
 }
 
-gchar *
+const gchar *
 dia_file_selector_get_file(DiaFileSelector *fs)
 {
-  /* UTF8 conversions here ? */
   return gtk_entry_get_text(GTK_ENTRY(fs->entry));
 }
 
