@@ -27,7 +27,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -36,7 +35,7 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
@@ -75,12 +74,13 @@ typedef struct _Box {
 
 static real sadtbox_distance_from(Box *box, Point *point);
 static void sadtbox_select(Box *box, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void sadtbox_move_handle(Box *box, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* sadtbox_move_handle(Box *box, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void sadtbox_move(Box *box, Point *to);
-static void sadtbox_draw(Box *box, Renderer *renderer);
+static ObjectChange* sadtbox_move(Box *box, Point *to);
+static void sadtbox_draw(Box *box, DiaRenderer *renderer);
 static void sadtbox_update_data(Box *box, AnchorShape horix, AnchorShape vert);
 static Object *sadtbox_create(Point *startpoint,
 			  void *user_data,
@@ -207,16 +207,17 @@ sadtbox_distance_from(Box *box, Point *point)
 
 static void
 sadtbox_select(Box *box, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   text_set_cursor(box->text, clicked_point, interactive_renderer);
   text_grab_focus(box->text, &box->element.object);
   element_update_handles(&box->element);
 }
 
-static void
+static ObjectChange*
 sadtbox_move_handle(Box *box, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
 
@@ -224,7 +225,7 @@ sadtbox_move_handle(Box *box, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&box->element, handle->id, to, reason);
+  element_move_handle(&box->element, handle->id, to, cp, reason, modifiers);
 
   switch (handle->id) {
   case HANDLE_RESIZE_NW:
@@ -247,19 +248,24 @@ sadtbox_move_handle(Box *box, Handle *handle,
     break;
   }
   sadtbox_update_data(box, horiz, vert);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 sadtbox_move(Box *box, Point *to)
 {
   box->element.corner = *to;
   
   sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
+
+  return NULL;
 }
 
 static void
-sadtbox_draw(Box *box, Renderer *renderer)
+sadtbox_draw(Box *box, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point lr_corner,pos;
   Element *elem;
   real idfontheight;
@@ -272,18 +278,18 @@ sadtbox_draw(Box *box, Renderer *renderer)
   lr_corner.x = elem->corner.x + elem->width;
   lr_corner.y = elem->corner.y + elem->height;
 
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->fill_rect(renderer, 
 			   &elem->corner,
 			   &lr_corner, 
 			   &SADTBOX_BG_COLOR);
 
 
-  renderer->ops->set_linewidth(renderer, SADTBOX_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, SADTBOX_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &elem->corner,
 			   &lr_corner, 
 			   &SADTBOX_FG_COLOR);
@@ -292,11 +298,11 @@ sadtbox_draw(Box *box, Renderer *renderer)
   text_draw(box->text, renderer);
 
   idfontheight = .75 * box->text->height;
-  renderer->ops->set_font(renderer, box->text->font, idfontheight);
+  renderer_ops->set_font(renderer, box->text->font, idfontheight);
   pos = lr_corner;
   pos.x -= .3 * idfontheight;
   pos.y -= .3 * idfontheight;
-  renderer->ops->draw_string(renderer,
+  renderer_ops->draw_string(renderer,
 			     box->id,
 			     &pos, ALIGN_RIGHT,
 			     &box->text->color);
@@ -348,7 +354,7 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   p = elem->corner;
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 - box->text->height * box->text->numlines / 2 +
-    font_ascent(box->text->font, box->text->height);
+    box->text->ascent;
   text_set_position(box->text, &p);
 
   extra->border_trans = SADTBOX_LINE_WIDTH / 2.0;
@@ -471,7 +477,8 @@ sadtbox_create(Point *startpoint,
   Element *elem;
   Object *obj;
   Point p;
-
+  DiaFont* font;
+  
   box = g_malloc0(sizeof(Box));
   elem = &box->element;
   obj = &elem->object;
@@ -490,20 +497,16 @@ sadtbox_create(Point *startpoint,
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 + /*default_properties.font_size*/ 0.8 / 2;
 
-  /*  box->text = new_text("", default_properties.font,
-		       default_properties.font_size, &p, 
-		       &SADTBOX_FG_COLOR,
-		       ALIGN_CENTER);
-  */
-
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. */
-  box->text = new_text("", font_getfont (_("Helvetica-Bold")),
-		       0.8, &p, 
-		       &color_black,
-		       ALIGN_CENTER);
+  font = dia_font_new_from_style( DIA_FONT_SANS|DIA_FONT_BOLD ,0.8);
+  
+  box->text = new_text("", font,
+                       0.8, &p, 
+                       &color_black,
+                       ALIGN_CENTER);
+  dia_font_unref(font);
+  
   box->id = g_strdup("A0"); /* should be made better. 
-				   Automatic counting ? */
+                               Automatic counting ? */
 
   element_init(elem, 8, 0);
 
