@@ -27,7 +27,7 @@
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -44,9 +44,9 @@ struct _LargePackage {
 
   ConnectionPoint connections[8];
 
-  utfchar *name;
-  utfchar *stereotype; /* Can be NULL, including << and >> */
-  utfchar *st_stereotype; 
+  char *name;
+  char *stereotype; /* Can be NULL, including << and >> */
+  char *st_stereotype; 
 
   DiaFont *font;
   
@@ -59,11 +59,12 @@ struct _LargePackage {
 
 static real largepackage_distance_from(LargePackage *pkg, Point *point);
 static void largepackage_select(LargePackage *pkg, Point *clicked_point,
-				Renderer *interactive_renderer);
-static void largepackage_move_handle(LargePackage *pkg, Handle *handle,
-				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void largepackage_move(LargePackage *pkg, Point *to);
-static void largepackage_draw(LargePackage *pkg, Renderer *renderer);
+				DiaRenderer *interactive_renderer);
+static ObjectChange* largepackage_move_handle(LargePackage *pkg, Handle *handle,
+					      Point *to, ConnectionPoint *cp,
+					      HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* largepackage_move(LargePackage *pkg, Point *to);
+static void largepackage_draw(LargePackage *pkg, DiaRenderer *renderer);
 static Object *largepackage_create(Point *startpoint,
 				   void *user_data,
 				   Handle **handle1,
@@ -124,6 +125,9 @@ static PropDescription largepackage_props[] = {
 static PropDescription *
 largepackage_describe_props(LargePackage *largepackage)
 {
+  if (largepackage_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(largepackage_props);
+  }
   return largepackage_props;
 }
 
@@ -161,14 +165,15 @@ largepackage_distance_from(LargePackage *pkg, Point *point)
 
 static void
 largepackage_select(LargePackage *pkg, Point *clicked_point,
-		    Renderer *interactive_renderer)
+		    DiaRenderer *interactive_renderer)
 {
   element_update_handles(&pkg->element);
 }
 
-static void
+static ObjectChange*
 largepackage_move_handle(LargePackage *pkg, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+			 Point *to, ConnectionPoint *cp,
+			 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(pkg!=NULL);
   assert(handle!=NULL);
@@ -176,21 +181,26 @@ largepackage_move_handle(LargePackage *pkg, Handle *handle,
 
   assert(handle->id < 8);
   
-  element_move_handle(&pkg->element, handle->id, to, reason);
+  element_move_handle(&pkg->element, handle->id, to, cp, reason, modifiers);
   largepackage_update_data(pkg);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 largepackage_move(LargePackage *pkg, Point *to)
 {
   pkg->element.corner = *to;
 
   largepackage_update_data(pkg);
+
+  return NULL;
 }
 
 static void
-largepackage_draw(LargePackage *pkg, Renderer *renderer)
+largepackage_draw(LargePackage *pkg, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point p1, p2;
@@ -205,48 +215,50 @@ largepackage_draw(LargePackage *pkg, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, LARGEPACKAGE_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, LARGEPACKAGE_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
 
   p1.x = x; p1.y = y;
   p2.x = x+w; p2.y = y+h;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
 			   &color_white);
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
 			   &color_black);
 
   p1.x= x; p1.y = y - pkg->topheight;
   p2.x = x + pkg->topwidth; p2.y = y;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
 			   &color_white);
   
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
 			   &color_black);
 
 
-  renderer->ops->set_font(renderer, pkg->font, LARGEPACKAGE_FONTHEIGHT);
+  renderer_ops->set_font(renderer, pkg->font, LARGEPACKAGE_FONTHEIGHT);
 
   p1.x = x + 0.1;
-  p1.y = y - LARGEPACKAGE_FONTHEIGHT - font_descent(pkg->font, LARGEPACKAGE_FONTHEIGHT) - 0.1;
+  p1.y = y - LARGEPACKAGE_FONTHEIGHT -
+      dia_font_descent(pkg->st_stereotype,
+                       pkg->font, LARGEPACKAGE_FONTHEIGHT) - 0.1;
 
 
 
-  if (pkg->st_stereotype) {
-    renderer->ops->draw_string(renderer, pkg->st_stereotype, &p1,
+  if (pkg->st_stereotype && pkg->st_stereotype[0] != '\0') {
+    renderer_ops->draw_string(renderer, pkg->st_stereotype, &p1,
 			       ALIGN_LEFT, &color_black);
   }
   p1.y += LARGEPACKAGE_FONTHEIGHT;
 
   if (pkg->name)
-    renderer->ops->draw_string(renderer, pkg->name, &p1,
+    renderer_ops->draw_string(renderer, pkg->name, &p1,
 			       ALIGN_LEFT, &color_black);
 }
 
@@ -258,18 +270,22 @@ largepackage_update_data(LargePackage *pkg)
 
   pkg->stereotype = remove_stereotype_from_string(pkg->stereotype);
   if (!pkg->st_stereotype) {
-    pkg->st_stereotype =  string_to_stereotype(pkg->stereotype);
+    pkg->st_stereotype = string_to_stereotype(pkg->stereotype);
   }
   
+  pkg->topheight = LARGEPACKAGE_FONTHEIGHT + 0.1*2;
+
   pkg->topwidth = 2.0;
   if (pkg->name != NULL)
     pkg->topwidth = MAX(pkg->topwidth,
-                        font_string_width(pkg->name, pkg->font,
+                        dia_font_string_width(pkg->name, pkg->font,
                                           LARGEPACKAGE_FONTHEIGHT)+2*0.1);
-  if (pkg->st_stereotype != NULL)
+  if (pkg->st_stereotype != NULL && pkg->st_stereotype[0] != '\0') {
     pkg->topwidth = MAX(pkg->topwidth,
-                        font_string_width(pkg->st_stereotype, pkg->font,
-                                          LARGEPACKAGE_FONTHEIGHT)+2*0.1);
+                        dia_font_string_width(pkg->st_stereotype, pkg->font,
+                                              LARGEPACKAGE_FONTHEIGHT)+2*0.1);
+    pkg->topheight += LARGEPACKAGE_FONTHEIGHT;
+  }
 
   if (elem->width < (pkg->topwidth + 0.2))
     elem->width = pkg->topwidth + 0.2;
@@ -293,6 +309,15 @@ largepackage_update_data(LargePackage *pkg)
   pkg->connections[7].pos.x = elem->corner.x + elem->width;
   pkg->connections[7].pos.y = elem->corner.y + elem->height;
   
+  pkg->connections[0].directions = DIR_NORTH|DIR_WEST;
+  pkg->connections[1].directions = DIR_NORTH;
+  pkg->connections[2].directions = DIR_NORTH|DIR_EAST;
+  pkg->connections[3].directions = DIR_WEST;
+  pkg->connections[4].directions = DIR_EAST;
+  pkg->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  pkg->connections[6].directions = DIR_SOUTH;
+  pkg->connections[7].directions = DIR_SOUTH|DIR_EAST;
+                                                                                          
   element_update_boundingbox(elem);
   /* fix boundingbox for top rectangle: */
   obj->bounding_box.top -= pkg->topheight;
@@ -321,6 +346,8 @@ largepackage_create(Point *startpoint,
 
   obj->ops = &largepackage_ops;
 
+  obj->can_parent = TRUE; 
+
   elem->corner = *startpoint;
 
   element_init(elem, 8, 8);
@@ -328,13 +355,12 @@ largepackage_create(Point *startpoint,
   elem->width = 4.0;
   elem->height = 4.0;
   
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. if "Courier" works for you, it would be better.  */
-  pkg->font = font_getfont (_("Courier"));
-
+  pkg->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,
+                                      LARGEPACKAGE_FONTHEIGHT);
+  
   pkg->stereotype = NULL;
   pkg->st_stereotype = NULL;
-  pkg->name = strdup("");
+  pkg->name = g_strdup("");
 
   pkg->topwidth = 2.0;
   pkg->topheight = LARGEPACKAGE_FONTHEIGHT*2 + 0.1*2;
@@ -348,13 +374,14 @@ largepackage_create(Point *startpoint,
   largepackage_update_data(pkg);
 
   *handle1 = NULL;
-  *handle2 = obj->handles[0];
+  *handle2 = obj->handles[7];
   return &pkg->element.object;
 }
 
 static void
 largepackage_destroy(LargePackage *pkg)
 {
+  dia_font_unref(pkg->font);
   g_free(pkg->stereotype);
   g_free(pkg->st_stereotype);
   g_free(pkg->name);
