@@ -30,7 +30,7 @@
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -57,11 +57,12 @@ static const double NODE_TEXT_MARGIN = 0.5;
 
 static real node_distance_from(Node *node, Point *point);
 static void node_select(Node *node, Point *clicked_point,
-				Renderer *interactive_renderer);
-static void node_move_handle(Node *node, Handle *handle,
-				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void node_move(Node *node, Point *to);
-static void node_draw(Node *node, Renderer *renderer);
+				DiaRenderer *interactive_renderer);
+static ObjectChange* node_move_handle(Node *node, Handle *handle,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* node_move(Node *node, Point *to);
+static void node_draw(Node *node, DiaRenderer *renderer);
 static Object *node_create(Point *startpoint,
 				   void *user_data,
 				   Handle **handle1,
@@ -125,6 +126,9 @@ static PropDescription node_props[] = {
 static PropDescription *
 node_describe_props(Node *node)
 {
+  if (node_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(node_props);
+  }
   return node_props;
 }
 
@@ -164,16 +168,17 @@ node_distance_from(Node *node, Point *point)
 
 static void
 node_select(Node *node, Point *clicked_point,
-		    Renderer *interactive_renderer)
+		    DiaRenderer *interactive_renderer)
 {
   text_set_cursor(node->name, clicked_point, interactive_renderer);
   text_grab_focus(node->name, &node->element.object);
   element_update_handles(&node->element);
 }
 
-static void
+static ObjectChange*
 node_move_handle(Node *node, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		 Point *to, ConnectionPoint *cp,
+		 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(node!=NULL);
   assert(handle!=NULL);
@@ -181,11 +186,13 @@ node_move_handle(Node *node, Handle *handle,
 
   assert(handle->id < 8);
   
-  element_move_handle(&node->element, handle->id, to, reason);
+  element_move_handle(&node->element, handle->id, to, cp, reason, modifiers);
   node_update_data(node);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 node_move(Node *node, Point *to)
 {
   Point p;
@@ -198,10 +205,13 @@ node_move(Node *node, Point *to)
   text_set_position(node->name, &p);
 
   node_update_data(node);
+
+  return NULL;
 }
 
-static void node_draw(Node *node, Renderer *renderer)
+static void node_draw(Node *node, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point points[4];
@@ -217,40 +227,40 @@ static void node_draw(Node *node, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, NODE_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, NODE_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   points[0].x = x;     points[0].y = y;
   points[1].x = x + w; points[1].y = y + h;
 
-  renderer->ops->fill_rect(renderer, points, points + 1, &color_white);
-  renderer->ops->draw_rect(renderer, points, points + 1, &color_black);
+  renderer_ops->fill_rect(renderer, points, points + 1, &color_white);
+  renderer_ops->draw_rect(renderer, points, points + 1, &color_black);
 
   points[1].x = x + NODE_DEPTH, points[1].y = y - NODE_DEPTH;
   points[2].x = x + w + NODE_DEPTH, points[2].y = y - NODE_DEPTH;
   points[3].x = x + w, points[3].y = y;
 
-  renderer->ops->fill_polygon(renderer, points, 4, &color_white);
-  renderer->ops->draw_polygon(renderer, points, 4, &color_black);
+  renderer_ops->fill_polygon(renderer, points, 4, &color_white);
+  renderer_ops->draw_polygon(renderer, points, 4, &color_black);
 
   points[0].x = points[3].x, points[0].y = points[3].y;
   points[1].x = points[0].x + NODE_DEPTH, points[1].y = points[0].y - NODE_DEPTH;
   points[2].x = points[0].x + NODE_DEPTH, points[2].y = points[0].y - NODE_DEPTH + h;
   points[3].x = points[0].x, points[3].y = points[0].y + h;
 
-  renderer->ops->fill_polygon(renderer, points, 4, &color_white);
-  renderer->ops->draw_polygon(renderer, points, 4, &color_black);
+  renderer_ops->fill_polygon(renderer, points, 4, &color_white);
+  renderer_ops->draw_polygon(renderer, points, 4, &color_black);
 
   text_draw(node->name, renderer);
   
-  renderer->ops->set_linewidth(renderer, NODE_LINEWIDTH);
+  renderer_ops->set_linewidth(renderer, NODE_LINEWIDTH);
   points[0].x = node->name->position.x;
   points[0].y = points[1].y = node->name->position.y + node->name->descent;
   for (i = 0; i < node->name->numlines; i++)
     { 
       points[1].x = points[0].x + node->name->row_width[i];
-      renderer->ops->draw_line(renderer, points, points + 1, &color_black);
+      renderer_ops->draw_line(renderer, points, points + 1, &color_black);
       points[0].y = points[1].y += node->name->height;
     }
 }
@@ -280,20 +290,28 @@ node_update_data(Node *node)
   
   /* Update connections: */
   node->connections[0].pos = elem->corner;
+  node->connections[0].directions = DIR_NORTH|DIR_WEST;
   node->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
   node->connections[1].pos.y = elem->corner.y;
+  node->connections[1].directions = DIR_NORTH;
   node->connections[2].pos.x = elem->corner.x + elem->width;
   node->connections[2].pos.y = elem->corner.y;
+  node->connections[2].directions = DIR_NORTH|DIR_EAST;
   node->connections[3].pos.x = elem->corner.x;
   node->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
+  node->connections[3].directions = DIR_WEST;
   node->connections[4].pos.x = elem->corner.x + elem->width;
   node->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
+  node->connections[4].directions = DIR_EAST;
   node->connections[5].pos.x = elem->corner.x;
   node->connections[5].pos.y = elem->corner.y + elem->height;
+  node->connections[5].directions = DIR_SOUTH|DIR_WEST;
   node->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
   node->connections[6].pos.y = elem->corner.y + elem->height;
+  node->connections[6].directions = DIR_SOUTH;
   node->connections[7].pos.x = elem->corner.x + elem->width;
   node->connections[7].pos.y = elem->corner.y + elem->height;
+  node->connections[7].directions = DIR_SOUTH|DIR_EAST;
   
   element_update_boundingbox(elem);
   /* fix boundingbox for depth: */
@@ -323,15 +341,15 @@ static Object *node_create(Point *startpoint, void *user_data, Handle **handle1,
   obj->ops = &node_ops;
 
   elem->corner = *startpoint;
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. */
-  font = font_getfont (_("Helvetica"));
+
+  font = dia_font_new_from_style (DIA_FONT_SANS, 0.8);
   /* The text position is recalculated later */
   p.x = 0.0;
   p.y = 0.0;
   node->name = new_text("", font, 0.8, &p, &color_black, ALIGN_LEFT);
   text_get_attributes(node->name,&node->attrs);
-
+  dia_font_unref(font);
+  
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -343,7 +361,7 @@ static Object *node_create(Point *startpoint, void *user_data, Handle **handle1,
   node_update_data(node);
 
   *handle1 = NULL;
-  *handle2 = obj->handles[0];
+  *handle2 = obj->handles[7];
   return &node->element.object;
 }
 
