@@ -27,7 +27,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -36,7 +35,7 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
@@ -76,12 +75,13 @@ typedef struct _Chronoref {
 
 static real chronoref_distance_from(Chronoref *chronoref, Point *point);
 static void chronoref_select(Chronoref *chronoref, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void chronoref_move_handle(Chronoref *chronoref, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* chronoref_move_handle(Chronoref *chronoref, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void chronoref_move(Chronoref *chronoref, Point *to);
-static void chronoref_draw(Chronoref *chronoref, Renderer *renderer);
+static ObjectChange* chronoref_move(Chronoref *chronoref, Point *to);
+static void chronoref_draw(Chronoref *chronoref, DiaRenderer *renderer);
 static void chronoref_update_data(Chronoref *chronoref);
 static Object *chronoref_create(Point *startpoint,
 			  void *user_data,
@@ -231,33 +231,40 @@ chronoref_distance_from(Chronoref *chronoref, Point *point)
 
 static void
 chronoref_select(Chronoref *chronoref, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   element_update_handles(&chronoref->element);
 }
 
-static void
+static ObjectChange*
 chronoref_move_handle(Chronoref *chronoref, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   g_assert(chronoref!=NULL);
   g_assert(handle!=NULL);
   g_assert(to!=NULL);
 
-  element_move_handle(&chronoref->element, handle->id, to, reason);
+  element_move_handle(&chronoref->element, handle->id, to, cp, 
+		      reason, modifiers);
   chronoref_update_data(chronoref);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 chronoref_move(Chronoref *chronoref, Point *to)
 {
   chronoref->element.corner = *to;
   chronoref_update_data(chronoref);
+
+  return NULL;
 }
 
 static void
-chronoref_draw(Chronoref *chronoref, Renderer *renderer)
+chronoref_draw(Chronoref *chronoref, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   Point lr_corner;
   real t;
@@ -267,8 +274,8 @@ chronoref_draw(Chronoref *chronoref, Renderer *renderer)
 
   elem = &chronoref->element;
   
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
 
   lr_corner.x = elem->corner.x + elem->width;
@@ -276,11 +283,11 @@ chronoref_draw(Chronoref *chronoref, Renderer *renderer)
 
   p1.y = p2.y = elem->corner.y;
 
-  renderer->ops->set_font(renderer, chronoref->font, chronoref->font_size);
+  renderer_ops->set_font(renderer, chronoref->font, chronoref->font_size);
   p3.y = p2.y + chronoref->majgrad_height + 
-    font_ascent(chronoref->font, chronoref->font_size);
+    dia_font_ascent("1",chronoref->font, chronoref->font_size);
 
-  renderer->ops->set_linewidth(renderer, chronoref->light_lwidth);
+  renderer_ops->set_linewidth(renderer, chronoref->light_lwidth);
   if (chronoref->time_lstep > 0.0) {
     p2.y = p1.y + chronoref->mingrad_height;
     for (t = chronoref->firstmaj, p1.x = chronoref->firstmin_x; 
@@ -288,11 +295,11 @@ chronoref_draw(Chronoref *chronoref, Renderer *renderer)
 	 t += chronoref->time_lstep, p1.x += chronoref->mingrad) {
       p2.x = p1.x;
     
-      renderer->ops->draw_line(renderer,&p1,&p2,&chronoref->color);
+      renderer_ops->draw_line(renderer,&p1,&p2,&chronoref->color);
     }
   }
 
-  renderer->ops->set_linewidth(renderer, chronoref->main_lwidth);
+  renderer_ops->set_linewidth(renderer, chronoref->main_lwidth);
   if (chronoref->time_step > 0.0) {
     p2.y = p1.y + chronoref->majgrad_height;
 
@@ -302,9 +309,9 @@ chronoref_draw(Chronoref *chronoref, Renderer *renderer)
       char time[10];
       p3.x = p2.x = p1.x;
     
-      renderer->ops->draw_line(renderer,&p1,&p2,&chronoref->color);
+      renderer_ops->draw_line(renderer,&p1,&p2,&chronoref->color);
       g_snprintf(time,sizeof(time),chronoref->spec,t);
-      renderer->ops->draw_string(renderer,time,&p3,ALIGN_CENTER,
+      renderer_ops->draw_string(renderer,time,&p3,ALIGN_CENTER,
 				 &chronoref->font_color);
     }
   }
@@ -312,7 +319,7 @@ chronoref_draw(Chronoref *chronoref, Renderer *renderer)
   p2.x = lr_corner.x;
   p1.y = p2.y = elem->corner.y;
 
-  renderer->ops->draw_line(renderer,&p1,&p2,&chronoref->color);
+  renderer_ops->draw_line(renderer,&p1,&p2,&chronoref->color);
 }
 
 static void
@@ -343,8 +350,8 @@ chronoref_update_data(Chronoref *chronoref)
   g_snprintf(biglabel,sizeof(biglabel),chronoref->spec,
 	   MIN(-ABS(chronoref->start_time),-ABS(chronoref->end_time)));
   
-  labelwidth = font_string_width(biglabel,chronoref->font,
-				 chronoref->font_size);
+  labelwidth = dia_font_string_width(biglabel,chronoref->font,
+                                     chronoref->font_size);
 
   /* Now, update the drawing helper counters */
   time_span = chronoref->end_time - chronoref->start_time;
@@ -429,9 +436,7 @@ chronoref_create(Point *startpoint,
 
   element_init(elem, 8, 0);
 
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. */
-  chronoref->font = font_getfont (_("Helvetica"));
+  chronoref->font = dia_font_new_from_style (DIA_FONT_SANS,1.0);
   chronoref->font_size = 1.0;
   chronoref->font_color = color_black;
   chronoref->start_time = 0.0;
@@ -452,6 +457,7 @@ chronoref_create(Point *startpoint,
 static void
 chronoref_destroy(Chronoref *chronoref)
 {
+  dia_font_unref(chronoref->font);
   connpointline_destroy(chronoref->scale);
   element_destroy(&chronoref->element);
 }
